/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("net.jazz.ajax.core.internal.IEUriMonitor");

dojo.require("net.jazz.ajax.core.internal.Uri");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");

//
// Consts
//

var IFRAME_ID = "net.jazz.ajax.iframe";

//
// Aliases
//

var checkNotNull   = net.jazz.ajax.toolbox.lang.checkNotNull;
var checkNotEmpty  = net.jazz.ajax.toolbox.lang.checkNotEmpty;

var setUriFragment = net.jazz.ajax.core.internal.setUriFragment;
var setUriQuery    = net.jazz.ajax.core.internal.setUriQuery;

//
// Utility functions
//

function createBackgroundIframe() {
	var iFrame = document.createElement("iframe");
	
	var uri = new dojo._Url(location.href);

	iFrame.id = IFRAME_ID;
	iFrame.src = dojo.moduleUrl("net.jazz.ajax", "ui/internal/Blank.html" + (uri.fragment ? "?" + uri.fragment : ""));
	iFrame.style.display = "none";
	document.body.appendChild(iFrame);
	
	return iFrame;
}

function IEUriMonitor(onLocationChangeFn) {
	// summary:
	//		Determine if the browser's URI has changed or if the user has
	//		pressed the back or forward button. If so, call the
	//		onLocationChangeFn with the new URI.
	//	description:
	//		Similar to UriMonitor, the primary difference being that IE doesn't
	//		add changes to the URI's hash into the history unless the hash
	//		value corresponds to an actual named anchor in the document, which
	//		is not how we use hash URIs in Jazz. To get around this IE
	//		difference, we use a background IFrame to maintain a back-forward
	//		history, by updating the IFrame's query string to correspond to the
	//		value of the main browser location's hash value.
	//		
	//		E.g. if the value of the browser window's location changes to
	//
	//		#action=jazz.viewPage&id=com.ibm.team.repository.admin
	//
	//		... then we'd update the IFrame's source to:
	//
	//		?action=jazz.viewPage&id=com.ibm.team.repository.admin
	//
	//		This design leads to a somewhat complex state machine, which is
	//		described below:
	//
	//		s1: Stable state - neither the window's location has changed nor
	//			has the IFrame's location. Note that this is the 99.9% case, so
	//			we optimize for it.
	//			Transitions: s1, s2, s3
	//		s2: Window's location changed - when a user clicks a hyperlink or
	//			code programmatically changes the window's URI.
	//			Transitions: s4
	//		s3: IFrame's location changed as a result of user pressing back or
	//			forward - when the user presses back or forward, the location of
	//			the background's IFrame changes to the previous or next value in
	//			its history.
	//			Transitions: s1
	//		s4: IEUriMonitor has programmatically changed the location of the
	//			background IFrame, but it's location hasn't yet changed. In this
	//			case we do nothing because we need to wait for the IFrame's
	//			location to reflect its actual state.
	//			Transitions: s4, s5
	//		s5:	IEUriMonitor has programmatically changed the location of the
	//			background IFrame, and the IFrame's location has caught up with
	//			reality. In this case we need to transition to s1.
	//			Transitions: s1
	// 
	checkNotNull(onLocationChangeFn, "onLocationChangeFn");
	
	createBackgroundIframe();

	// shortcuts to the objects we'll be using a lot
	var iframeWindow = window[IFRAME_ID];
	var iframeNode = dojo.byId(IFRAME_ID);
	var iframeLoc = iframeWindow.location;
	var winLoc = window.location;
	
	// declare vars in outer function so that they're shared by inner functions
	var recentHash, recentIframeQuery, isIframeTransitioning, expectedIFrameQuery;

	function resetState() {
		recentHash = winLoc.hash;
		recentIframeQuery = iframeLoc.search;
		isIframeTransitioning = false;
		expectedIFrameQuery = null;
	}
	
	// initialize state (transition to s1)
	resetState();
	
	this.pollLocation = function() {
		
		// check to see if we're in an IFrame location transition (s4 or s5)
		// 
		// Note: recentHash check is a workaround for Defect 30120
		// TODO: Revisit algorithm to verify correctness in all edge cases
		if(isIframeTransitioning && recentHash === winLoc.hash) {
			// if the IFrame's location has caught up (s5), transition back to
			// the stable state (s1)
			if (iframeLoc.search === expectedIFrameQuery) {
				resetState();
			}
			// whether we were in s4 or s5, return
			return;
		}
		
		// if we're still in a stable state (s1), do nothing
		if(recentHash === winLoc.hash &&
				recentIframeQuery === iframeLoc.search) {
			return;
		}
		
		// if we're still going, either the window's location has changed (s2)
		// or the IFrame's location has changed as a result of the user clicking
		// the back or forward button (s3)
		if(recentHash !== winLoc.hash) {
			
			// workaround for Defect 30120
			recentHash = winLoc.hash;
			
			// s2 (window location changed)
			var locationUri = new dojo._Url(winLoc.href);
			// transition to s4
			isIframeTransitioning = true;
			expectedIFrameQuery = setUriQuery(iframeLoc.search, locationUri.fragment || "");
			iframeNode.src = setUriQuery(iframeNode.src, locationUri.fragment || "");
		} else {
			// s3 (iframe's location changed via user action)
			var iframeUri = new dojo._Url(iframeLoc.href);
			// update the browser's URI
			winLoc.href = setUriFragment(winLoc.href, iframeUri.query || "");
			// transition to s1
			resetState();
		}
		onLocationChangeFn(new dojo._Url(winLoc.href));
		return;
	}
	
	setInterval(this.pollLocation, 200);
}

net.jazz.ajax.core.internal.IEUriMonitor = IEUriMonitor;

})();